#!/usr/bin/env python3

import re
import sys
import os
import shutil
import stat
import subprocess

is_bad_lib = re.compile(r'(/usr/local|/opt)').match
is_sys_lib = re.compile(r'(/usr|/System)').match
otool_libname_extract = re.compile(r'\s+([/@].*?)[(\s:]').search
goodlist = []
badlist = []
badlist_orig = []
link_map = {}


def otool(cmdline):
    with subprocess.Popen(['otool'] + cmdline, stdout=subprocess.PIPE,
                          encoding='utf-8') as p:
        return p.stdout.readlines()


def get_rpath(lib):
    info = otool(['-l', lib])
    commands = []
    command = []
    for line in info:
        line = line.strip()
        if line.startswith("Load command "):
            commands.append(command)
            command = []
        else:
            command.append(line)
    commands.append(command)

    # yuck
    return [line.split()[1] for command in commands if "cmd LC_RPATH" in command for line in command if line.startswith("path")]


def collectlibs(lib, masterlist, targetdir):
    global goodlist, link_map
    liblist = otool(['-L', lib])
    locallist = []

    for l in liblist:
        l = otool_libname_extract(l)
        if not l:
            continue

        l = l.group(1)
        l_orig = l

        if l.startswith("@rpath/"):
            rpath = get_rpath(lib)

            if not rpath:
                print(f"{lib} uses @rpath but has no rpath set!")
                exit(-1)

            # all cases of libs using rpath so far just had a single directory in rpath...
            # so let's just hope it stays that way and worry about the other cases when we get to them
            if len(rpath) >= 2:
                print(f"Warning: {lib} uses @rpath with more than one entry in rpath. Guessing one entry...")

            l = os.path.join(rpath[0], l[len("@rpath/"):])

        if l.startswith("@loader_path/"):
            l = os.path.join(os.path.dirname(lib), l[len("@loader_path/"):])

        if is_bad_lib(l):
            if l not in badlist:
                badlist.append(l)
            if l_orig not in badlist_orig:
                badlist_orig.append(l_orig)
        if ((not is_sys_lib(l)) or is_bad_lib(l)) and l not in masterlist:
            locallist.append(l)
            print("found %s:" % l)

            check = l
            link_list = []
            while check:
                basename = os.path.basename(check)
                target = os.path.join(targetdir, basename)

                if os.path.islink(target):
                    # If a library was a symlink to a file with the same name in another directory,
                    # this could otherwise cause a FileNotFoundError
                    os.remove(target)

                if os.path.isfile(check) and not os.path.islink(check):
                    try:
                        shutil.copy(check, target)
                    except PermissionError:
                        print("    FILE %s ... skipped" % check)
                        break
                    print("    FILE %s ... copied to target" % check)
                    if link_list:
                        for link in link_list:
                            link_map[link] = basename
                    break

                if os.path.islink(check):
                    link_dst = os.readlink(check)
                    try:
                        os.symlink(link_dst, target)
                    except FileExistsError:
                        print("    LINK %s ... existed" % check)
                        break
                    print("    LINK %s ... copied to target" % check)
                    link_list.append(basename)
                    check = os.path.join(os.path.dirname(check), link_dst)
        elif l not in goodlist and l not in masterlist:
            goodlist.append(l)
    masterlist.extend(locallist)

    for l in locallist:
        collectlibs(l, masterlist, targetdir)


if __name__ == '__main__':
    binname = sys.argv[1]
    targetdir = os.path.dirname(binname)
    print("Searching for libraries in", binname, "...")
    libs = [binname]
    collectlibs(binname, libs, targetdir)

    print()
    print("System libraries used...")
    goodlist.sort()
    for l in goodlist:
        print(l)

    print()
    print("Fixing library install names...")
    in_tool_cmdline = ['install_name_tool']
    for lib in badlist_orig:
        libbase = os.path.basename(lib)
        if libbase in link_map:
            print("%s -> @executable_path/%s (REMAPPED)" % (lib, libbase))
            libbase = link_map[libbase]
        else:
            print("%s -> @executable_path/%s" % (lib, libbase))
        in_tool_cmdline = in_tool_cmdline + ['-change', lib,
                                             '@executable_path/' + libbase]
    for lib in libs:
        libbase = os.path.basename(lib)

        if libbase in link_map:
            libbase = link_map[libbase]

        targetlib = targetdir + '/' + libbase
        orig_permission = os.stat(targetlib).st_mode
        if not(orig_permission & stat.S_IWUSR):
            os.chmod(targetlib, orig_permission | stat.S_IWUSR)
        subprocess.run(in_tool_cmdline + ['-id', '@executable_path/' + libbase,
                                          targetlib])
        if not(orig_permission & stat.S_IWUSR):
            os.chmod(targetlib, orig_permission)

    if badlist:
        print()
        print("WARNING: The following libraries have blacklisted paths:")
        for lib in sorted(badlist):
            print(lib)
        print(
            "These paths normally have files from a package manager, which means that end result may not work if copied to another machine.")

    print()
    print("All done!")



#!/usr/bin/env python3

import sys, subprocess, pathlib, os

mt_exe, executable = sys.argv[1:]

subprocess.run([mt_exe, '-manifest', executable + '.manifest', '-outputresource:' + executable + ';1'])
pathlib.Path(os.path.join(os.path.dirname(executable), 'applied_manifest')).touch()



# Copyright (c) 2014, Thomas Goyne <plorkyeran@aegisub.org>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
# Aegisub Project http://www.aegisub.org/

# A script to strip all of the data we don't use out of ICU's data files
# Run from $ICU_ROOT/icu4c/source/data

from __future__ import unicode_literals
import re
import os

# Remove stuff we don't use at all from the Makefile
def delete_matching(filename, strs):
    exprs = [re.compile(s) for s in strs]

    with open(filename, encoding='utf-8') as f:
        lines = [line for line in f if not any(r.match(line) for r in exprs)]

    with open(filename, 'w', encoding='utf-8') as f:
        for line in lines:
            f.write(line)

REMOVE_SUBDIRS=['LOCSRCDIR', 'CURRSRCDIR', 'ZONESRCDIR', 'COLSRCDIR', 'RBNFSRCDIR', 'TRANSLITSRCDIR']
delete_matching('Makefile.in', ['^-include .*%s' % s for s in REMOVE_SUBDIRS])
delete_matching('Makefile.in', ['^CNV_FILES'])

with open('misc/misclocal.mk', 'w', encoding='utf-8') as f:
    f.write('MISC_SOURCE = supplementalData.txt likelySubtags.txt icuver.txt icustd.txt metadata.txt')

# Remove data we don't need from the lang and region files
def parse_txt(filename):
    root = {}
    cur = root
    stack = [root]
    comment = False
    for line in open(filename, encoding='utf-8'):
        line = line.strip()
        if len(line) == 0:
            continue
        if '//' in line:
            continue
        if '/*' in line:
            comment = True
            continue
        if comment:
            if '*/' in line:
                comment = False
            continue

        if line == '}':
            stack.pop()
            cur = stack[-1]
            continue
        if line.endswith('{'):
            obj = {}
            cur[line[:-1]] = obj
            cur = obj
            stack.append(obj)
            continue

        m = re.match('(.*){"(.*)"}', line)
        if m:
            cur[m.group(1)] = m.group(2)

    return root

def remove_sections(root):
    for child in root.values():
        child.pop('Keys', None)
        child.pop('LanguagesShort', None)
        child.pop('Types', None)
        child.pop('Variants', None)
        child.pop('codePatterns', None)
        child.pop('localeDisplayPattern', None)
        child.pop('CountriesShort', None)
        child.pop('Scripts%stand-alone', None)

def remove_languages(root):
    for lang, child in root.items():
        # We only care about a language's name in that language
        lang = lang.split('_')[0]
        trimmed = {}
        v = child.get('Languages', {}).get(lang)
        if v:
            trimmed[lang] = v
        child['Languages'] = trimmed

# Scripts which are actually used by stuff
SCRIPTS = ['Cyrl', 'Latn', 'Arab', 'Vaii', 'Hans', 'Hant']
def remove_scripts(root):
    for lang, child in root.items():
        v = child.get('Scripts')
        if not v:
            continue

        trimmed = {}
        for script in SCRIPTS:
            if v.get(script):
                trimmed[script] = v[script]
        child['Scripts'] = trimmed

def write_dict(name, value, out, indent):
    if len(value) == 0:
        return

    child_indent = indent + '    '

    out.write(indent)
    out.write(name)
    out.write('{\n')
    for k in sorted(value.keys()):
        v = value[k]
        if type(v) == dict:
            write_dict(k, v, out, child_indent)
        else:
            out.write(('%s%s{"%s"}\n' % (child_indent, k, v)))
    out.write(indent)
    out.write('}\n')

def write_file(root, filename):
    with open(filename, 'w', encoding='utf-8') as f:
        for k, v in root.items():
            write_dict(k, v, f, '')

def minify_lang(filename):
    f = parse_txt(filename)
    remove_sections(f)
    remove_languages(f)
    remove_scripts(f)
    write_file(f, filename)

for name in os.listdir('lang'):
    if not name.endswith('.txt'):
        continue
    minify_lang('lang/' + name)

# gather information about which language+region combinations actually exist,
# so that we can drop all others
def gather_regions():
    langs = {
        'af': ['ZA'],
        'am': ['ET'],
        'ar': ['AE', 'BH', 'DZ', 'EG', 'IQ', 'JO', 'KW', 'LB', 'LY', 'MA', 'OM', 'QA', 'SA', 'SY', 'TN', 'YE'],
        'arn': ['CL'],
        'as': ['IN'],
        'az': ['AZ', 'AZ'],
        'ba': ['RU'],
        'be': ['BY'],
        'bg': ['BG'],
        'bn': ['BD', 'IN'],
        'bo': ['CN'],
        'br': ['FR'],
        'bs': ['BA', 'BA'],
        'ca': ['ES'],
        'co': ['FR'],
        'cs': ['CZ'],
        'cy': ['GB'],
        'da': ['DK'],
        'de': ['AT', 'CH', 'DE', 'LI', 'LU'],
        'div': ['MV'],
        'el': ['GR'],
        'en': ['029', 'AU', 'BZ', 'CA', 'GB', 'IE', 'IN', 'JM', 'MY', 'NZ', 'PH', 'SG', 'TT', 'US', 'ZA', 'ZW'],
        'es': ['AR', 'BO', 'CL', 'CO', 'CR', 'DO', 'EC', 'ES', 'GT', 'HN', 'MX', 'NI', 'PA', 'PE', 'PR', 'PY', 'SV', 'US', 'UY', 'VE'],
        'et': ['EE'],
        'eu': ['ES'],
        'fa': ['IR'],
        'fi': ['FI'],
        'fil': ['PH'],
        'fo': ['FO'],
        'fr': ['BE', 'CA', 'CH', 'FR', 'LU', 'MC'],
        'fy': ['NL'],
        'ga': ['IE'],
        'gl': ['ES'],
        'gsw': ['FR'],
        'gu': ['IN'],
        'ha': ['NG'],
        'he': ['IL'],
        'hi': ['IN'],
        'hr': ['BA', 'HR'],
        'hu': ['HU'],
        'hy': ['AM'],
        'id': ['ID'],
        'ig': ['NG'],
        'ii': ['CN'],
        'is': ['IS'],
        'it': ['CH', 'IT'],
        'iu': ['CA', 'CA'],
        'ja': ['JP'],
        'ka': ['GE'],
        'kk': ['KZ'],
        'kl': ['GL'],
        'km': ['KH'],
        'kn': ['IN'],
        'ko': ['KR'],
        'kok': ['IN'],
        'ky': ['KG'],
        'lb': ['LU'],
        'lo': ['LA'],
        'lt': ['LT'],
        'lv': ['LV'],
        'mi': ['NZ'],
        'mk': ['MK'],
        'ml': ['IN'],
        'mn': ['CN', 'MN'],
        'moh': ['CA'],
        'mr': ['IN'],
        'ms': ['BN', 'MY'],
        'mt': ['MT'],
        'nb': ['NO'],
        'ne': ['NP'],
        'nl': ['BE', 'NL'],
        'nn': ['NO'],
        'nso': ['ZA'],
        'oc': ['FR'],
        'or': ['IN'],
        'pa': ['IN'],
        'pl': ['PL'],
        'prs': ['AF'],
        'ps': ['AF'],
        'pt': ['BR', 'PT'],
        'qut': ['GT'],
        'quz': ['BO', 'EC', 'PE'],
        'rm': ['CH'],
        'ro': ['RO'],
        'ru': ['RU'],
        'rw': ['RW'],
        'sa': ['IN'],
        'sah': ['RU'],
        'se': ['FI', 'NO', 'SE'],
        'si': ['LK'],
        'sk': ['SK'],
        'sl': ['SI'],
        'sma': ['NO', 'SE'],
        'smj': ['NO', 'SE'],
        'smn': ['FI'],
        'sms': ['FI'],
        'sq': ['AL'],
        'sr': ['BA', 'BA', 'SP', 'YU'],
        'sv': ['FI', 'SE'],
        'sw': ['KE', 'TZ'],
        'syr': ['SY'],
        'ta': ['IN'],
        'te': ['IN'],
        'tg': ['TJ'],
        'th': ['TH'],
        'tk': ['TM'],
        'tn': ['ZA'],
        'tr': ['TR'],
        'tt': ['RU'],
        'tzm': ['DZ'],
        'ug': ['CN'],
        'uk': ['UA'],
        'ur': ['PK'],
        'uz': ['UZ', 'UZ'],
        'vi': ['VN'],
        'wee': ['DE'],
        'wen': ['DE'],
        'wo': ['SN'],
        'xh': ['ZA'],
        'yo': ['NG'],
        'zh': ['CN', 'HK', 'MO', 'SG', 'TW'],
        'zu': ['ZA']
    }
    for name in os.listdir('region'):
        if not name.endswith('.txt'): continue
        parts = name[:-4].split('_')
        if len(parts) == 1: continue
        if not parts[0] in langs:
            langs[parts[0]] = []
        langs[parts[0]].extend(parts[1:])
    return langs

REGIONS = gather_regions()
def remove_countries(root):
    for lang, child in root.items():
        v = child.get('Countries', {})
        if not v: continue

        # We only care about the names for regions in the languages used in
        # those regions
        lang = lang.split('_')[0]
        regions = REGIONS.get(lang)
        if not regions:
            del child['Countries']
            continue

        trimmed = {}
        for region in regions:
            name = v.get(region)
            if name:
                trimmed[region] = name
        child['Countries'] = trimmed

def minify_region(filename):
    f = parse_txt(filename)
    remove_sections(f)
    remove_countries(f)
    write_file(f, filename)

for name in os.listdir('region'):
    if not name.endswith('.txt'):
        continue
    minify_region('region/' + name)




#!/usr/bin/env python3

import sys, os

manifestfile, cppfile, hfile = sys.argv[1:]

with open(manifestfile, 'r') as manifest:
    files = dict((x.strip(), None) for x in manifest.readlines() if x.strip() != '')

sourcepath = os.path.split(manifestfile)[0]
buildpath = os.path.split(cppfile)[0]

for k in files:
    sf = os.path.join(sourcepath, k)
    bf = os.path.join(buildpath, k)

    if os.path.isfile(sf):
        files[k] = sf
    elif os.path.isfile(bf):
        files[k] = bf
    else:
        print("{}: Failed to open '{}'".format(manifestfile, k))
        sys.exit(1)

with open(cppfile, 'w') as cpp:
    cpp.write('#include "libresrc.h"\n')
    with open(hfile, 'w') as h:

        for k in files:
            with open(files[k], 'rb') as f:
                data = [str(int(x)) for x in f.read()]

            datastr = ','.join(data)
            name = os.path.splitext(os.path.basename(k))[0]
            cpp.write('const unsigned char {}[] = {{{}}};\n'.format(name, datastr))
            h.write('extern const unsigned char {}[{}];\n'.format(name, len(data)))
